package awais.instagrabber.activities;

import android.content.Intent;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.widget.ArrayAdapter;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.recyclerview.widget.RecyclerView;

import com.thoughtbot.expandablerecyclerview.models.ExpandableGroup;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import awais.instagrabber.R;
import awais.instagrabber.adapters.FollowCompareAdapter;
import awais.instagrabber.adapters.FollowsAdapter;
import awais.instagrabber.asyncs.FollowFetcher;
import awais.instagrabber.interfaces.FetchListener;
import awais.instagrabber.models.FollowModel;
import awais.instagrabber.utils.Utils;

public class FollowViewer extends AppCompatActivity {
    private String id, name, namePost;
    private boolean followers;
    private Toolbar toolbar;
    private RecyclerView rvFollow;
    private View.OnClickListener clickListener;
    private boolean isCompare = false;
    private FollowModel model;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_follow);

        rvFollow = findViewById(R.id.rvFollow);
        toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        final Intent intent = getIntent();
        if (intent == null) {
            Utils.errorFinish(this);
            return;
        }

        followers = intent.getBooleanExtra("followers", false);
        id = intent.getStringExtra("id");
        if (Utils.isEmpty(id)) {
            Utils.errorFinish(this);
            return;
        }

        name  = intent.getStringExtra("name");
        namePost = name + " is";
        if (Utils.isEmpty(name)) {
            name = "You";
            namePost = "You're";
        }

        final ArrayAdapter<Object> adapter = new ArrayAdapter<>(this, android.R.layout.simple_list_item_1, new String[]{"View profile", "Open in Instagram"});
        final AlertDialog alertDialog = new AlertDialog.Builder(this).setAdapter(adapter, (dialog, which) -> {
            if (model != null) {
                if (which == 0) {
                    if (Main.scanHack != null) {
                        Main.scanHack.onResult(model.getUsername());
                        finish();
                    }
                } else {
                    final Intent actIntent = new Intent(Intent.ACTION_VIEW, Uri.parse("https://instagram.com/" + model.getUsername()));
                    if (Utils.isInstagramInstalled) actIntent.setPackage("com.instagram.android");
                    startActivity(actIntent);
                }
            }
        }).setTitle("What to do?").create();

        clickListener = v -> {
            final Object tag = v.getTag();
            if (tag instanceof FollowModel) {
                model = (FollowModel) tag;
                if (!alertDialog.isShowing()) alertDialog.show();
            }
        };

        listFollows();
    }

    private void listFollows() {
        toolbar.setTitle(followers ? "Followers" : "Following");

        final ArrayList<FollowModel> followModels = new ArrayList<>();
        final FollowsAdapter followsAdapter = new FollowsAdapter(followModels, clickListener);
        rvFollow.setAdapter(followsAdapter);

        final FetchListener<FollowModel[]> fetchListener = new FetchListener<FollowModel[]>() {
            @Override
            public void onResult(final FollowModel[] result) {
                if (result != null) {
                    final int oldSize = followModels.size();
                    followModels.addAll(Arrays.asList(result));

                    followsAdapter.notifyItemRangeInserted(oldSize, result.length);

                    final FollowModel model = result[result.length - 1];
                    if (model != null && model.hasNextPage()) {
                        new FollowFetcher(id, followers, model.getEndCursor(), this).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                        model.setPageCursor(false, null);
                    }
                }
            }
        };

        new FollowFetcher(id, followers, fetchListener).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }

    private void listCompare() {
        toolbar.setTitle("Compare followers & following");

        final ArrayList<FollowModel> followingModels = new ArrayList<>();
        final ArrayList<FollowModel> followersModels = new ArrayList<>();
        final ArrayList<FollowModel> allFollowing = new ArrayList<>();

        final FetchListener<FollowModel[]> followingFetchListener = new FetchListener<FollowModel[]>() {
            @Override
            public void onResult(final FollowModel[] result) {
                if (result != null) {
                    followingModels.addAll(Arrays.asList(result));

                    final FollowModel model = result[result.length - 1];
                    if (model != null && model.hasNextPage()) {
                        new FollowFetcher(id, false, model.getEndCursor(), this).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                        model.setPageCursor(false, null);
                    } else {
                        allFollowing.addAll(followersModels);
                        allFollowing.retainAll(followingModels);

                        for (final FollowModel followModel : allFollowing) {
                            followersModels.remove(followModel);
                            followingModels.remove(followModel);
                        }

                        allFollowing.trimToSize();
                        followersModels.trimToSize();
                        followingModels.trimToSize();

                        final List<ExpandableGroup<FollowModel>> groups = new ArrayList<>(1);
                        if (followingModels.size() > 0)
                            groups.add(new ExpandableGroup<>("not following " + name, followingModels));
                        if (followersModels.size() > 0)
                            groups.add(new ExpandableGroup<>(namePost + " not following", followersModels));
                        if (allFollowing.size() > 0)
                            groups.add(new ExpandableGroup<>("Both following each other", allFollowing));

                        rvFollow.setAdapter(new FollowCompareAdapter(FollowViewer.this, clickListener, groups));
                        Log.d("AWAISKING_APP", "second: " + followersModels.size() + " -- " + followingModels.size());
                    }
                }
            }
        };
        final FetchListener<FollowModel[]> followersFetchListener = new FetchListener<FollowModel[]>() {
            @Override
            public void onResult(final FollowModel[] result) {
                if (result != null) {
                    followersModels.addAll(Arrays.asList(result));
                    final FollowModel model = result[result.length - 1];
                    if (model == null || !model.hasNextPage())
                        new FollowFetcher(id, false, followingFetchListener).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                    else {
                        new FollowFetcher(id, true, model.getEndCursor(), this).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                        model.setPageCursor(false, null);
                    }
                }
            }
        };

        new FollowFetcher(id, true, followersFetchListener).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }

    @Override
    public boolean onCreateOptionsMenu(final Menu menu) {
        getMenuInflater().inflate(R.menu.follow, menu);

        menu.getItem(0).setOnMenuItemClickListener(item -> {
            rvFollow.setAdapter(null);
            if (isCompare) listFollows();
            else listCompare();
            isCompare = !isCompare;
            return true;
        });

        return true;
    }
}