package awais.instagrabber.activities;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.annotation.SuppressLint;
import android.app.NotificationManager;
import android.content.ClipboardManager;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.database.MatrixCursor;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.PersistableBundle;
import android.os.Process;
import android.provider.BaseColumns;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.ContentFrameLayout;
import androidx.appcompat.widget.SearchView;
import androidx.appcompat.widget.Toolbar;
import androidx.fragment.app.FragmentManager;
import androidx.recyclerview.widget.RecyclerView;

import java.net.CookieHandler;
import java.util.ArrayList;

import awais.instagrabber.BuildConfig;
import awais.instagrabber.R;
import awais.instagrabber.adapters.SuggestionsAdapter;
import awais.instagrabber.asyncs.SuggestionsAsync;
import awais.instagrabber.interfaces.FetchListener;
import awais.instagrabber.models.ClipModel;
import awais.instagrabber.models.PostModel;
import awais.instagrabber.models.ProfileModel;
import awais.instagrabber.models.StoryModel;
import awais.instagrabber.models.SuggestionModel;
import awais.instagrabber.utils.CircularImageView;
import awais.instagrabber.utils.FlavorTown;
import awais.instagrabber.utils.MainHelper;
import awais.instagrabber.utils.MyApps;
import awais.instagrabber.utils.Utils;
import awais.instagrabber.utils.dialogs.SettingsDialog;

import static awais.instagrabber.utils.Utils.clipboardManager;
import static awais.instagrabber.utils.Utils.notificationManager;
import static awais.instagrabber.utils.Utils.startClipboardMonitor;
import static awais.instagrabber.utils.dialogs.SettingConstants.BOTTOM_SNACK;

public class Main extends AppCompatActivity {
    public static SharedPreferences sharedPreferences;
    public static FetchListener<String> scanHack;
    public static final ArrayList<PostModel> allItems = new ArrayList<>();
    public final ArrayList<PostModel> selectedItems = new ArrayList<>();
    private AsyncTask<String, String, SuggestionModel[]> prevAsync;
    private SuggestionsAdapter suggestionAdapter;
    private MenuItem searchAction;
    public Toolbar toolbar;
    public SearchView searchView;
    public RecyclerView mainPosts;
    public MenuItem downloadAction;
    public StoryModel[] storyModels;
    public View privatePage, progressCircular, highlightsParent, isVerified;
    public LinearLayout highlightsList;
    public CircularImageView mainProfileImage;
    public TextView mainPostCount, mainFollowers, mainFollowing, mainFullName, mainBiography;
    public String userQuery = null;
    public MainHelper mainHelper;
    public ProfileModel profileModel;
    private AlertDialog profileClickDialog;
    private ArrayAdapter<String> profileDialogAdapter;
    private DialogInterface.OnClickListener profileDialogListener;

    @Override
    protected void onCreate(final Bundle bundle) {
        super.onCreate(bundle);
        setContentView(R.layout.activity_main);

        FlavorTown.updateCheck(this);

        CookieHandler.setDefault(Utils.NET_COOKIE_MANAGER);

        Utils.isInstagramInstalled = Utils.isInstaInstalled(this);

        if (sharedPreferences == null)
            sharedPreferences = getSharedPreferences("settings", MODE_PRIVATE);
        if (notificationManager == null)
            notificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
        if (clipboardManager == null)
            clipboardManager = (ClipboardManager) getSystemService(Context.CLIPBOARD_SERVICE);

        final String cookie = sharedPreferences.getString("cookie", null);
        Utils.setupCookies(cookie);

        mainHelper = new MainHelper(this);
        userQuery = bundle == null ? null : bundle.getString("query");

        scanHack = result -> {
            if (mainHelper != null && !Utils.isEmpty(result)) {
                userQuery = result;
                mainHelper.doSearch();
            }
        };

        suggestionAdapter = new SuggestionsAdapter(this, v -> {
            final Object tag = v.getTag();
            if (tag instanceof CharSequence) {
                userQuery = tag.toString();
                mainHelper.doSearch();
            }
            if (searchView != null && !searchView.isIconified()) {
                if (searchAction != null) searchAction.collapseActionView();
                searchView.setIconified(true);
                searchView.setIconified(true);
            }
        });

        profileDialogAdapter = new ArrayAdapter<>(this, android.R.layout.simple_list_item_1, new String[]{"Show profile picture", "Show stories"});
        profileDialogListener = (dialog, which) -> {
            final Intent intent;
            if (which == 0 || storyModels == null || storyModels.length < 1)
                intent = new Intent(this, ProfileViewer.class).putExtra("profile", profileModel);
            else intent = new Intent(this, StoryViewer.class).putExtra("username", userQuery)
                    .putExtra("stories", storyModels);
            startActivity(intent);
        };
        initProfileDialog();

        final ContentFrameLayout parent = findViewById(android.R.id.content);
        final ViewGroup sliderView = (ViewGroup) LayoutInflater.from(this).inflate(R.layout.clip_slider, parent, false);
        final TextView sliderTextView = (TextView) sliderView.getChildAt(0);
        final int sliderHeight = Utils.convertDpToPx(64);
        final Handler handler = new Handler();
        final Runnable runnable = new Runnable() {
            @Override
            public void run() {
                sliderView.clearAnimation();
                sliderView.animate().translationY(Main.sharedPreferences.getBoolean(BOTTOM_SNACK, true) ? sliderHeight : -sliderHeight).setListener(new AnimatorListenerAdapter() {
                    @Override
                    public void onAnimationEnd(final Animator animation) {
                        sliderView.clearAnimation();
                    }
                }).setDuration(700).start();
                handler.removeCallbacks(this);
            }
        };
        startClipboardMonitor(new FetchListener<ClipModel>() {
            private boolean bottom_snack = Main.sharedPreferences.getBoolean(BOTTOM_SNACK, true);
            private ClipModel clipModel;

            @SuppressLint("SetTextI18n")
            @Override
            public void onResult(final ClipModel model) {
                this.clipModel = model;

                if (model != null && parent != null && progressCircular != null && progressCircular.getVisibility() == View.GONE) {
                    bottom_snack = Main.sharedPreferences.getBoolean(BOTTOM_SNACK, true);

                    runOnUiThread(() -> {
                        if (parent.getChildCount() == 1) {
                            if (bottom_snack)
                                sliderView.setLayoutParams(new FrameLayout.LayoutParams(-1, sliderHeight, Gravity.BOTTOM));
                            parent.addView(sliderView, parent.getChildCount());
                            sliderTextView.setOnClickListener(v -> {
                                handler.removeCallbacks(runnable);
                                sliderView.clearAnimation();
                                sliderView.setTranslationY(bottom_snack ? sliderHeight : -sliderHeight);
                                if (clipModel.isPost()) {
                                    startActivity(new Intent(Main.this, PostViewer.class)
                                            .putExtra("post", new PostModel(false, null, null,
                                                    null, clipModel.getText(), null, 0))
                                            .putExtra("user", userQuery));
                                } else {
                                    userQuery = clipModel.getText();
                                    mainHelper.doSearch();
                                }
                            });
                        }
                        sliderTextView.setText("A " + (clipModel.isPost() ? "post link" : "username") + " has been copied to clipboard!\nTap here to view it.");
                        sliderView.setTranslationY(bottom_snack ? sliderHeight : -sliderHeight);
                        sliderView.animate().translationY(0).setDuration(700).setListener(new AnimatorListenerAdapter() {
                            @Override
                            public void onAnimationEnd(final Animator animation) {
                                handler.postDelayed(runnable, 3017);
                            }
                        }).start();
                    });
                }
            }
        });

        final View.OnClickListener onClickListener = v -> {
            if (v == mainBiography) {
                Utils.copyText(this, mainBiography.getText().toString());
            } else if (v == mainProfileImage) {
                if (storyModels == null || storyModels.length <= 0) {
                    profileDialogListener.onClick(null, 0);
                } else {
                    if (profileClickDialog == null) initProfileDialog();
                    profileClickDialog.show();
                }
            }
        };

        mainBiography.setOnClickListener(onClickListener);
        mainProfileImage.setOnClickListener(onClickListener);

        mainBiography.setEnabled(false);
        mainProfileImage.setEnabled(false);

        if (BuildConfig.DEBUG) {
            userQuery = "the.badak"; // todo
            mainHelper.doSearch();
        }

        mainHelper.onIntent(getIntent());
    }

    @Override
    protected void onNewIntent(final Intent intent) {
        super.onNewIntent(intent);
        mainHelper.onIntent(intent);
    }

    private void initProfileDialog() {
        if (profileClickDialog == null)
            profileClickDialog = new AlertDialog.Builder(this).setAdapter(profileDialogAdapter, profileDialogListener)
                    .setNeutralButton("Cancel", null).create();
    }

    @Override
    public void onSaveInstanceState(@NonNull final Bundle outState, @NonNull final PersistableBundle outPersistentState) {
        outState.putString("query", userQuery);
        super.onSaveInstanceState(outState, outPersistentState);
    }

    @Override
    public void onRestoreInstanceState(@Nullable final Bundle savedInstanceState, @Nullable final PersistableBundle persistentState) {
        super.onRestoreInstanceState(savedInstanceState, persistentState);
        if (savedInstanceState != null) userQuery = savedInstanceState.getString("query");
    }

    @Override
    protected void onSaveInstanceState(@NonNull final Bundle outState) {
        outState.putString("query", userQuery);
        super.onSaveInstanceState(outState);
    }

    @Override
    protected void onRestoreInstanceState(@NonNull final Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);
        userQuery = savedInstanceState.getString("query");
    }

    @Override
    public boolean onCreateOptionsMenu(final Menu menu) {
        getMenuInflater().inflate(R.menu.menu, menu);

        final FragmentManager fragmentManager = getSupportFragmentManager();
        final MenuItem.OnMenuItemClickListener clickListener = item -> {
            if (item == downloadAction) Utils.batchDownload(this);
            else new SettingsDialog().show(fragmentManager, "settings");
            return true;
        };

        menu.findItem(R.id.action_settings).setVisible(true)
                .setOnMenuItemClickListener(clickListener);

        searchAction = menu.findItem(R.id.action_search);
        downloadAction = menu.findItem(R.id.action_download);

        searchView = (SearchView) searchAction.getActionView();
        downloadAction.setOnMenuItemClickListener(clickListener);

        searchView.setQueryHint("Search username...");
        searchView.setSuggestionsAdapter(suggestionAdapter);
        searchView.setOnSearchClickListener(v -> searchView.setQuery(userQuery, false));
        searchView.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            private final String[] COLUMNS = {BaseColumns._ID, "username", "type", "pfp", "verified"};

            @Override
            public boolean onQueryTextSubmit(final String query) {
                userQuery = query;
                searchAction.collapseActionView();
                searchView.setIconified(true);
                searchView.setIconified(true);
                mainHelper.doSearch();
                return false;
            }

            @Override
            public boolean onQueryTextChange(final String newText) {
                //todo suggestions in next update
                if (prevAsync != null) try { prevAsync.cancel(true); } catch (final Exception ignored) { }

                if (!Utils.isEmpty(newText) && newText.length() > 2) {
                    prevAsync = new SuggestionsAsync(new FetchListener<SuggestionModel[]>() {
                        @Override
                        public void onBefore() {
                            suggestionAdapter.changeCursor(null);
                        }

                        @Override
                        public void onResult(final SuggestionModel[] result) {
                            if (result == null) {
                                suggestionAdapter.changeCursor(null);
                            } else {
                                final MatrixCursor cursor = new MatrixCursor(COLUMNS);
                                for (int i = 0; i < result.length; i++) {
                                    final SuggestionModel suggestionModel = result[i];
                                    if (suggestionModel != null)
                                        cursor.addRow(new Object[]{i, suggestionModel.getUsername(), suggestionModel.getSuggestionType(), suggestionModel.getProfilePic(),
                                                suggestionModel.isVerified()});
                                }
                                suggestionAdapter.changeCursor(cursor);
                            }
                        }
                    }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, newText);
                }
                return true;
            }
        });

        return true;
    }

    @Override
    public void onBackPressed() {
        if (searchView != null && !searchView.isIconified()) {
            if (searchAction != null) searchAction.collapseActionView();
            searchView.setIconified(true);
            searchView.setIconified(true);
            return;
        }
        if (mainHelper.isSelectionCleared()) {
            MyApps.showAlertDialog(this, (parent, view, position, id) -> {
                if (id == -1 && position == -1 && parent == null) Process.killProcess(Process.myPid());
                else MyApps.openAppStore(this, position);
            });
        }
    }

    @Override
    public void onRequestPermissionsResult(final int requestCode, @NonNull final String[] permissions, @NonNull final int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == 17 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            Utils.batchDownload(this);
        }
    }
}